/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Maurizio Munafo', Marco Mellia, Gloria Bogliolo, Ana Paula Couto,
		  Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	Implementation of the AnaPeer subclass of Peer.
 */

#include "anapeer.h"

// ===================== AnaPeer ================================
// An experimental peer, based on LatestBAPeer, for use by polito only.

/// Chunk scheduler -- same as LatestBA (currently, but may change).

void AnaPeer::schedule_new_chunk()
{
	bool done = false;
	ChunkIterator cit = window()->begin();	// Iterator over chunks, latest first
	while (cit != window()->end() && !done && num_links_out() > 0 && !p_upload_busy) {
		PChunk chunk = *cit;
		PLink selected_link = NULL;

		std::vector<double> cdf;	// Cumulative Density Function
		std::vector<PLink> candidate_links;

		double total_weight = 0;

		// Loop through neighbours one at a time, checking if they need the current chunk
		// and aren't busy. If so, add them to a list of candidates and to a cumulative
		// density function.

		for (LinkIterator lit = first_out_link(); lit != last_out_link(); ++lit) {
			PLink link = *lit;
			PPeer neighbour = link->to();
			if (neighbour->needs_chunk(chunk) && !neighbour->download_busy()) {
				double wt = 1; // std::sqrt(neighbour->upload_bandwidth());
				total_weight += wt;
				cdf.push_back(total_weight);
				candidate_links.push_back(link);
			}
		}

		// Choose one of the candidates, using the cumulative density function to weight
		// the selection.

		if (!cdf.empty()) {
			double ran = uniform(0, total_weight, &gSeed);  // The "coin toss" selection value
			size_t selected_index = std::lower_bound(cdf.begin(), cdf.end(), ran) - cdf.begin();
			selected_link = candidate_links[selected_index];
		}

		if (selected_link != NULL) {
			schedule_chunk(chunk, selected_link);
			done = true;
		}
		else {  // Couldn't find a neighbour interested in that chunk. Move to next.
			++cit;
		}
	}
}
