/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file overlaytypes.cpp
    Defines the various concrete Overlay subtypes. The subclasses all have
	set_peers and set_edges methods that respectively define the
	overlay's peers and edges.
 */

#include <iostream>
#include "types.h"
#include "peer.h"
#include "asawareoverlay.h"
#include "misc.h"
#include "globals.h"
#include "propertymap.h"
#include "asnetwork.h"
#include "router.h"

// =================================================================================

// The ASAwareOverlay -- an as aware overlay.
// ----------------

ASAwareOverlay::ASAwareOverlay(
        size_t num_peers, ///< The number of peers in the overlay
        size_t degree ///< The number of random neighbours to choose.
        )
{
    if (degree > num_peers - 1)
    {
        fatal("The degree is larger than the total number of peers");
    }
    
    set_peers(num_peers);
    set_edges(degree);
}



void ASAwareOverlay::set_peers(size_t num_peers)
{
    PASNetwork net = (PASNetwork) Network::n_current_network;
    PPeer source = Peer::make_peer(0, 0);
    Host* sHost = Host::create_host(0, 0, 0, 0);
    PAS as = net->add_as();
    PRouter sRouter = new Router();
    net->assignRouter(sRouter, as);
    sRouter->add_host(sHost);
    source->set_host(sHost);
    add_peer(source);

    for (size_t i = 0; i < num_peers - 2; i++)
    {
        PPeer child = Peer::make_peer(2, 0);
        sHost = Host::create_host(2, 0, 0, 0);
        as = net->add_as();
        sRouter = new Router();
        net->assignRouter(sRouter, as);
        sRouter->add_host(sHost);
        child->set_host(sHost);
        add_peer(child);
    }
    PPeer tmp = Peer::make_peer(0, 5);
    sHost = Host::create_host(0, 5, 0, 0);
    as = net->add_as();
    sRouter = new Router();
    net->assignRouter(sRouter, as);
    sRouter->add_host(sHost);
    tmp->set_host(sHost);
    add_peer(tmp);

    double latency = config.getDouble("LATENCY_VALUE");
    std::cout << "Using latency " << latency << std::endl;
    for (size_t i = 0; i < num_peers; i++)
    {
        for (size_t j = 0; j < num_peers; j++)
        {
            //TODO: this should be specified in the config.txt
            if (i != j) net->linkAs(i, j, latency);

        }
    }

}

void ASAwareOverlay::set_edges(size_t degree)
{
    for (size_t i = 0; i < num_peers() - 1; i++)
    {
        for (size_t j = i + 1; j < num_peers(); j++)
        {
            add_link(i, j);
            add_link(j, i);
        }
    }
}

