#TESTH=$SRCDIRABS/test.h
SIMULATOR='$SRCDIRABS/sim'


function backend_init {
  recompile $1
}

function video_init {
  if [[ $PSNR_VIDEO ]]; then
    eval PSNR_CODER2=\"$PSNR_CODER\"
    $PSNR_TOOLS/QoE.pl init $PSNR_VIDEO "$PSNR_CODER2"
  fi
}


function configure_init {
  echo >/dev/null
}




function configure_scheduler {
  PEER_TYPE="CONFIG_PEER"
  
  unset SCHEDULER_PARAMETERS

  if [ $CSCHED = "LUc" ]; then
    CS="BestChunk,Recentness"
  elif [ $CSCHED = "RUc" ]; then
    CS="BestChunk,UnitWtChunk"
  elif [ $CSCHED = "DLc" ]; then
    CS="BestChunk,DLc"
    SCHEDULER_PARAMETERS="$SCHEDULER_PARAMETERS DLIncrement=2"
  elif [[ $CSCHED =~ '^DLc([0-9\.]+)$' ]] ; then
    CS="BestChunk,DLc"
    SCHEDULER_PARAMETERS="$SCHEDULER_PARAMETERS DLIncrement=${BASH_REMATCH[1]}"
  else
    echo "ERROR: unsupported chunk scheduler: $CSCHED"
    exit
   fi

  if [ $PSCHED = "BAp_w" ]; then
    PS="WeightedPeer,UploadBW" 
  elif [ $PSCHED = "BAp" ]; then
    PS="BestPeer,UploadBW" 
  elif [ $PSCHED = "ELp" ]; then
    PS="BestPeer,EarliestLatest"
  elif [ $PSCHED = "BAELp" ]; then
    PS="BestPeer,BAELp" 
  #elif [ $PSCHED = "U_LNNC_p" ];
  #elif [ $PSCHED = "MDp" ];
  #elif [ $PSCHED = "U_LC_p" ];
  #elif [ $PSCHED = "U_DOWNUP_p" ];
  #elif [ $PSCHED = "U_STRIPE_p" ];
  elif [ $PSCHED = "RUp" ]; then
    PS="WeightedPeer,NeedsSelected,UnitWtPeer" 
  else
    echo "ERROR: unsupported peer scheduler: $PSCHED"
    exit
  fi

  if [ $SCHED_ORDER = "C_P" ]; then
    # the filter should logically be UsefulToSomeone, but ChunkFirst selection loop anyway, so we can also use AllChunks. This will speed up things considerably!
    SCHEDULER_PARAMETERS="$SCHEDULER_PARAMETERS"\
" ConfigPeerConfig=ChunkFirst,$CS,AllChunks,$PS,NeedsSelected"\
" ConfigPeerSourceConfig=ChunkFirst,BestChunk,Recentness,AllChunks,$PS,NeedsSelected"\
" PlayoutDelay=$PLAYOUT_DEADLINE"\
" PeerType=$PEER_TYPE"

  elif [ $SCHED_ORDER = "P_C" ]; then
    SCHEDULER_PARAMETERS="$SCHEDULER_PARAMETERS"\
" ConfigPeerConfig=PeerFirst,$PS,NeedsAny,$CS,UsefulToSelected"\
" ConfigPeerSourceConfig=PeerFirst,$PS,NeedsAny,BestChunk,Recentness,UsefulToSelected"\
" PlayoutDelay=$PLAYOUT_DEADLINE"\
" PeerType=$PEER_TYPE"
  fi 
}

# configure_network $SRCDIRABS
function configure_network {
  NETWORK_PARAMETERS=""\
" NumPeers=$NUM_PEERS"\
" NumChunks=$NUM_CHUNKS"\
" StartStats=`echo a=$NUM_CHUNKS*$FIRST_CHUNK';a/1'|bc`"\
" StopStats=`echo a=$NUM_CHUNKS*$LAST_CHUNK';a/1-1'|bc`"\
" VideoRate=$VIDEORATE"\
" Degree=$DEGREE"\
" PeerBandwidths={{$PEERSCLASS1,$MEANCLASS1,$DOWNLOAD_BW},{$PEERSCLASS2,$MEANCLASS2,$DOWNLOAD_BW},{$PEERSCLASS3,$MEANCLASS3,$DOWNLOAD_BW},{$PEERSCLASS4,$MEANCLASS4,$DOWNLOAD_BW}}"\
" BandwidthFractVariation=$VARIANCERATE"\
" ProbChunkError=$PROB_CHUNK_ERROR"\
" ChunkTimeoutInterval=$CHUNK_TIMEOUT"\
" FECn=$FECN"\
" FECk=$FECK"\
""
}

function configure_other {
  if [[ $PERCENTILE ]]; then
    OTHER_PARAMETERS=""\
" DelayCdfFile=$DELAY_CDF_FILE"
  fi
}

function recompile {
  cd $1
  make
  cd -
}

# params: 1:SRCDIRABS 2:seed, 3:resultsdir
function run {
  CMD="$1/sim $NETWORK_PARAMETERS $SCHEDULER_PARAMETERS $OTHER_PARAMETERS OverlaySeed=$2"
  echo running: $CMD

  cd $3

  #we need config.txt here
  cp $1/config.script.txt config.txt || exit 1
  # we also need a folder called "output"
  mkdir -p output
  # create repeat script
  echo $CMD >rerun.sh
  chmod a+x rerun.sh

  $1/proc-time 2>&1 $CMD | tee run.out
  RESULTSLINE+=`grep TESTresults run.out`
  if [[ $PSNR_VIDEO ]]; then
    $PSNR_TOOLS/QoE.pl calc output/ $PSNR_EVERY >psnr.out
    RESULTSLINE+=" "`grep PSNRresults psnr.out`
  else
    RESULTSLINE+=" PSNRresults NaN NaN"
  fi
  RESULTSLINE+=" "`grep user run.out`
  RESULTSLINE+=" "`grep MAXSIZE run.out`
  RESULTSLINE+=" "`grep AVGSIZE run.out`
  cd -

  echo -e $RESULTSLINE >>$TESTOUT
  if [[ $OUTPREFIX ]]; then
    eval OUTPREFIX2=\"$OUTPREFIX\"
    cp $3/$DELAY_CDF_FILE.out $OUTPREFIX2-$DELAY_CDF_FILE.out
  fi
}


