
from numpy import arange, sin, pi
import numpy as np
import os

def printline(s):
    print s


def readNumbers(dir, filename, returnType="numpy"):
    fname = dir + "/" + filename;
    try:
        f = open(fname, "r")
    except IOError:
        f = None
    if f == None:
        print "File %s not found" % fname
        return None

    lines = f.readlines()
    data = []
    for line in lines:
        thisLine = []
        bits = line.split()
        for bit in bits:
            thisLine.append(float(bit))
        data.append(thisLine)
    if returnType == "numpy":
        values = np.array(data)
    else:
        values = data

    f.close()
    return values

def getDegrees(lis, maxClass):
    result = []
    for c in range(0,maxClass+1):
        data = np.array(lis[c])
        result.append(data.mean())
    return result

# Build a lookup-table of peer-classes by reading the given upload-rate file.
# Returns (maxClassNum, lookupTable) where lookupTable[i] is the class of peer #i
def getPeerClasses(filename):
    maxClass = -1
    try:
        peers_file = open(filename, "r")
    except IOError:
        print "File %s not found" % filename
        return
    lines = peers_file.readlines()
    peers_file.close()
    peer_class=[]
    for line in lines:
        bits = line.split()
        assert int(bits[0]) == len(peer_class)
        itsClass = int(bits[2])
        if itsClass > maxClass:
            maxClass = itsClass
        peer_class.append(itsClass)
    return (maxClass, peer_class)

# Return a array giving the number of times each peer appears in the peer_list
def get_frequencies(peer_list, numPeers):
    counts = [0 for peer in range(0, numPeers)]
    for p in peer_list:
        counts[p] += 1
    return counts

# Split the list freqs (frequencies of occurrence of each peer in some list),
# into a list of sublists, one for each class of peer according to the given
# classes array.
def classify(freqs, classes, maxClass):
    degrees = [[] for c in range(0, maxClass+1)]
    for i in range(0, len(classes)):
        degrees[classes[i]].append(freqs[i])
    return degrees

# Return a list of delay stats for the run in the given directory.
# List contains: loss%, meanDelay, medianDelay for all peers then for
# each class of peer.
def delayStats(dir):
    delays = readNumbers(dir, "DelayHistograms.out", returnType="list");
    stats = []
    if delays != None:
        numBars = int(delays[0][0])
        numPeers = int(delays[0][1])
        barWidth = delays[0][2]
        allDelays = np.zeros(numBars)
        totLosses = 0
        lossesByClass = {}
        delaysByClass = {}
        for peer in range(0, numPeers):
            (id, peer_class, lost) = delays[1 + peer*2]
            binCounts = np.array(delays[2 + peer*2])
            allDelays += binCounts
            totLosses += lost
            if peer_class in delaysByClass:
                delaysByClass[peer_class] += binCounts
                lossesByClass[peer_class] += lost
            else:
                delaysByClass[peer_class] = binCounts
                lossesByClass[peer_class] = lost
        stats = getStats(barWidth, totLosses, allDelays)
        for c in delaysByClass:
            stats += getStats(barWidth, lossesByClass[c], delaysByClass[c])
    return stats

# Support method for delayStats. Computes actual statistics given
# an array of histogram bin counts (assumed to start at zero) and a bin width.
# Returns a list [lossPercent, meanDelay, medianDelay, 95%-ile]
def getStats(binWidth, lost, bins):
    binCentre = binWidth/2
    sum = np.sum(bins)
    halfsum = sum/2
    min = None
    median = None
    max = 0
    percentile95 = None
    percent95 = sum*0.95
    totl = 0
    cnt = 0
    
    for bin in bins:
        if bin > 0:
            if min == None:
                min = binCentre
            max = binCentre
            cnt += bin
            totl += bin * binCentre
            if median == None and cnt >= halfsum:
                median = binCentre
            if percentile95 == None and cnt >= percent95:
                percentile95 = binCentre
        binCentre += binWidth
        
    stats = [100.0*lost/cnt, totl/cnt, median, percentile95]
    return stats

def processSubfolders(parentDir):
    dirs = [f for f in os.listdir(parentDir) if "Degree" in f]
    for dir in dirs:
        subDir = "%s/%s" % (parentDir, dir)
        delays = delayStats(subDir)
        print subDir,
        for dly in delays:
            print "%5.3f"%dly,
        print

import sys
if (len(sys.argv) != 2):
    print "Usage: batchStats folderName"
    sys.exit(1)
processSubfolders(sys.argv[1])



