/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

Authors:    Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
			Telecomunication Networks Group
			Politecnico di Torino
			Torino, Italy
			mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
    Defines the Chunk class. This represents a chunk of video information
	from the original stream, not to be confused with the CHUNK_TRANSMISSION
	class, which represents the various copies of this original chunk being
	sent around the network. 
 */

#ifndef _CHUNK_H_
#define _CHUNK_H_

#include "types.h"

/// The Chunk struct
class Chunk
{
	ChunkID c_id;			///< The ID number (sequence number) of the chunk
	Time c_initiation_time;	///< The time at which the chunk becomes available at the source
	bool c_parity;	///<marks a FEC parity chunk
        ChunkSize c_size; /// chunk size in bytes
        ChunkType c_type; /// chunk type (I, P..etc..)
public:
	/// The constructor for a chunk
	Chunk(size_t id, Time time, ChunkSize size, ChunkType type, bool par = false) : c_id(id), c_initiation_time(time), c_size(size), c_type(type), c_parity(par) {}

	/// Get the chunk ID
	inline ChunkID id() { return c_id; }

	/// Get the chunk initiation time
	inline Time initiation_time() { return c_initiation_time; }

	/// Get whether this is a FEC parity chunk
	inline bool parity() { return c_parity; }

        /// Get the chunk size in Mbit
        inline ChunkSize size() { return c_size; }

        /// Get the chunk type
        inline ChunkType type() { return c_type; }
};

#endif // _CHUNK_H_
