# set common default parameters below. These will be later overridden as follows:
#  testcase (-t) overrides first, if defined
#  testcasefile (-T) overrides second

# a lower case "s" at the end of the parameter name means that the parameter is a list, i.e. the script will iterate through all the given values

CYCLEs=1                # don't change, it does not make sense with the current code! seed guarantees deterministic behaviour, independent of the cycle
NUM_SEEDS=1             # number of random graphs to generate for the test. Note that MAX values will be averaged amond runs with different seeds
SEEDs=`seq 1 1 $NUM_SEEDS`
NUM_ITERATIONS=1        # not used in current code! would be the number of runs done (with each algorithm) on the same graph. MAX values will be maxed among these runs

NUM_PEERSs=100          # number of peers, including the source
GRAPH_MODEL=4           # 0:GNR 1:WS 2:GNP 3:TREE 4:GNKREG
DEGREEs='($NUM_PEERS-1)'   # neighborhood size. Set in to '($NUM_PEERS-1)' for a full mesh. To be improved when better graph generation algorithms are implemented
MEANBWs=1.0             # mean bandwidth. 1:critical regime; <1:overload regime >1:undeload regime
PEERSCLASS1s=1.0        # ratio of peers in the first class
MEANCLASS1s='(($MEANBW-(1.0-$PEERSCLASS1)*$MEANCLASS2)/$PEERSCLASS1)' # BW of peers in the first class. It can be calcualted based on MEANBW = PC1*MC1 + (1-PC1)*MC2
PEERSCLASS2s='(1-$PEERSCLASS1)'
MEANCLASS2s=0.5        

PEERSCLASS3s=0.0
MEANCLASS3s=0.0
PEERSCLASS4s=0.0
MEANCLASS4s=0.0

DOWNLOAD_BWs=INF        # download bandwidth default value
DOWN_MIN=0.1		# download bandwidth: min value (used in the 'download' testcase)
DOWN_MAX=1.0		# download bandwidth: max value (used in the 'download' testcase)
DOWN_INC=0.1		# download bandwidth: increment (used in the 'download' testcase)

VARIANCERATEs=0

VIDEORATEs=1
PSNR_CODER='-f m4v -mbd 2 -b ${VIDEORATE}M'

PROB_CHUNK_ERRORs=0
CHUNK_TIMEOUTs=-1

NUM_CHUNKSs=200         # number of chunks
FIRST_CHUNK=0.1
LAST_CHUNK=0.9
FECNs=0
FECKs=0

NON_CLAIRVOYANT=0	# 0: clairvoyant 1:non-clairvoyant
COLLISION_TIMEOUT=delta # the timeout at a peer if collision happens. use "delta" to have it at the chunk transmission time, use 0.1 to have it as absolute time independent of the bandwidth
SCHED_ORDERs="C_P P_C"  # chunk-first or peer-first scheduler
CSCHEDs="LUc RUc DLc"   #RUSc EUc         # chunk scheduling alogorithms used
PSCHEDs="ELp RUp BAp BAp_w BAELp"   #U_LC_p U_LNNUC_p BAp BAp_w U_DOWNUP_p U_STRIPE_p BAELp           # peer scheduling algorithms used
WIN_SIZEs="NUM_CHUNKS"  # trading window size. Set to "NUM_CHUNKS" to disable
PLAYOUT_DEADLINEs="32"  # playout deadline: chunk having more delay are not needed and dropped. Set to "-1" to disable

PLAYOUT_MIN=1		# playout delay: min value
PLAYOUT_MAX=1000	# playout delay: max value
PLAYOUT_INC=100		# playout delay: increment

DELAY_CDF_FILE="Perchunkinstance_cdf"	#.out added by code

VAR_Is=0 #generic variables to used for fast prototypes
VAR_Js=0
VAR_Ks=0

#==================================================================
#LINEHEAD='$RUN_UID\t$CYCLE\t$SEED\t$NUM_PEERS\t$DEGREE\t$MEANBW\t$PEERSCLASS1\t$MEANCLASS1\t$PEERSCLASS2\t$MEANCLASS2\t$PEERSCLASS3\t$MEANCLASS3\t$PEERSCLASS4\t$MEANCLASS4\t$VARIANCERATE\t$NUM_CHUNKS\t$SCHED_ORDER\t$CSCHED\t$PSCHED\t$WIN_SIZE\t$VAR_I\t$VAR_J\t$VAR_K' #will be evaluated later using eval
#==================================================================
function configure_testcase {
case $1 in 
  winsize)
    XLABEL="\"window size (in chunks)\""
    XSHORTNAME=winsize
    XINDEX=$(linehead_index WIN_SIZE)
    PLOTTITLE='Effect of window size \n
     homogenous bandwith, set to $MEANBWs \n
     peers=$NUM_PEERSs; degree=$DEGREEs; chunks=$NUM_CHUNKSs; iterations=$NUM_ITERATIONS; seeds=$NUM_SEEDS; window=$WIN_SIZEs'    #will be evaluated later using eval
    WIN_SIZEs="5 10 15 20 30 40 50 70 100 190 195 200"
    ;;

  highbwratio)
    XLABEL="\"ratio of high BW nodes (inhomogeneous)\""
    XSHORTNAME=highbwratio
    XINDEX=$(linehead_index PEERSCLASS1)
    PLOTTITLE='Effect of inhomogeneity \n
     average BW fixed at $MEANBWs, low BW fixed at $MEANCLASS2s \n
     peers=$NUM_PEERSs; degree=$DEGREEs; chunks=$NUM_CHUNKSs; iterations=$NUM_ITERATIONS; seeds=$NUM_SEEDS; window=$WIN_SIZEs'    #will be evaluated later using eval
    MEANBWs=1.0
    PEERSCLASS1s=`seq 0.05 0.05 1`
    MEANCLASS2s=0.5        
    ;;

  highbw)
    XLABEL="\"high BW (note that the number of such nodes decreases, the average is always the same)\""
    XSHORTNAME=highbw
    XINDEX=$(linehead_index MEANCLASS1)
    PLOTTITLE='Effect of inhomogeneity \n
     average BW fixed at $MEANBWs, low BW fixed at $MEANCLASS2s \n
     peers=$NUM_PEERSs; degree=$DEGREEs; chunks=$NUM_CHUNKSs; iterations=$NUM_ITERATIONS; seeds=$NUM_SEEDS; window=$WIN_SIZEs'    #will be evaluated later using eval
    MEANBWs=1.0
    PEERSCLASS1s=`seq 0.05 0.05 1`
    MEANCLASS2s=0.5        
    ;;

  bw)
    XLABEL="\"homogeneous BW\""
    XSHORTNAME=bw
    XINDEX=$(linehead_index MEANCLASS1)
    PLOTTITLE='Effect of homogeneous bandwidth\n
     peers=$NUM_PEERSs; degree=$DEGREEs; chunks=$NUM_CHUNKSs; iterations=$NUM_ITERATIONS; seeds=$NUM_SEEDS; window=$WIN_SIZEs'    #will be evaluated later using eval
    MEANBWs=`seq 0.9 0.10 3`
    PEERSCLASS1s=1.0
    ;;
              
  download)
    XLABEL="\"download BW [Mbit/s]\""
    XSHORTNAME=downbw
    XINDEX=$(linehead_index DOWNLOAD_BW)
    PLOTTITLE='Effect of download bandwidth\n
     peers=$NUM_PEERSs; degree=$DEGREEs; chunks=$NUM_CHUNKSs; iterations=$NUM_ITERATIONS; seeds=$NUM_SEEDS; window=$WIN_SIZEs'    #will be evaluated later using eval
    PEERSCLASS1s=0.3     #test values
    MEANCLASS1s=`expr 0.5*$MEANBWs | bc -l`
    PEERSCLASS2s=0.5
    MEANCLASS2s=`expr 1.0*$MEANBWs | bc -l`
    PEERSCLASS3s=0.1
    MEANCLASS3s=`expr 2.0*$MEANBWs | bc -l`
    PEERSCLASS4s=0.1
    MEANCLASS4s=`expr 2.0*$MEANBWs | bc -l`
    DOWNLOAD_BWs=`seq $DOWN_MIN $DOWN_INC $DOWN_MAX`
    ;;
    
 playout_delay)
    XLABEL="\"playout delay [s]\""
    XSHORTNAME=playdelay
    XINDEX=$(linehead_index PLAYOUT_DEADLINE)
    PLOTTITLE='Effect of playout delay\n
     peers=$NUM_PEERSs; degree=$DEGREEs; chunks=$NUM_CHUNKSs; iterations=$NUM_ITERATIONS; seeds=$NUM_SEEDS; mean upload=$MEANBWs'    #will be evaluated later using eval
    PEERSCLASS1s=0.3     #test values
    MEANCLASS1s=`expr 0.5*$MEANBWs | bc -l`
    PEERSCLASS2s=0.5
    MEANCLASS2s=`expr 1.0*$MEANBWs | bc -l`
    PEERSCLASS3s=0.1
    MEANCLASS3s=`expr 2.0*$MEANBWs | bc -l`
    PEERSCLASS4s=0.1
    MEANCLASS4s=`expr 2.0*$MEANBWs | bc -l`
    PLAYOUT_DEADLINEs=`seq $PLAYOUT_MIN $PLAYOUT_INC $PLAYOUT_MAX`
    ;;

  numnodes)
    XLABEL="\"Number of peers\""
    XSHORTNAME=numnodes
    XINDEX=$(linehead_index NUM_PEERS)
    PLOTTITLE='Number of peers\n
     homogenous bandwith, set to $MEANBWs \n
     peers=$NUM_PEERSs; degree=$DEGREEs; chunks=$NUM_CHUNKSs; iterations=$NUM_ITERATIONS; seeds=$NUM_SEEDS; window=$WIN_SIZEs'    #will be evaluated later using eval
    NUM_PEERSs="2 3 5 7 10 15 20 30 50 70 100"
    ;;

  degree)
    XLABEL="\"Neighborhood size\""
    XSHORTNAME=degree
    XINDEX=$(linehead_index DEGREE)
    PLOTTITLE='Neighborhood size\n
     homogenous bandwith, set to $MEANBWs \n
     peers=$NUM_PEERSs; degree=$DEGREEs; chunks=$NUM_CHUNKSs; iterations=$NUM_ITERATIONS; seeds=$NUM_SEEDS; window=$WIN_SIZEs'    #will be evaluated later using eval
    DEGREEs="1 2 3 4 5 7 10 15 20 30 50 70 99"
    ;;

  numchunks)
    XLABEL="\"Stream length (number of chunks)\""
    XSHORTNAME=numchunks
    XINDEX=$(linehead_index NUM_CHUNKS)
    PLOTTITLE='Stream length (number of chunks)\n
     homogenous bandwith, set to $MEANBWs \n
     peers=$NUM_PEERSs; degree=$DEGREEs; chunks=$NUM_CHUNKSs; iterations=$NUM_ITERATIONS; seeds=$NUM_SEEDS; window=$WIN_SIZEs'    #will be evaluated later using eval
    NUM_CHUNKSs="5 20 50 100 200 500 1000 5000"
    ;;

  chunkdelay-cdf)
    XLABEL="\"chunk delay\""
    XSHORTNAME=delay
    PLOTTITLE='CDf of individual chunk delays\n
     homogenous bandwith, set to $MEANBWs \n
     peers=$NUM_PEERSs; degree=$DEGREEs; chunks=$NUM_CHUNKSs; iterations=$NUM_ITERATIONS; seeds=$NUM_SEEDS; window=$WIN_SIZEs'    #will be evaluated later using eval
    OUTPREFIX='$CSCHED-$PSCHED'
    STAT_CDF=1
    ;;
  *)
    echo "wrong testcase. Look in $0 for more details :)"
    exit 1
    ;;

esac
}



