/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Maurizio Munafo', Marco Mellia, Richard Lobb, Gloria Bogliolo, 
/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

Authors:    Maurizio Munafo', Marco Mellia, Gloria Bogliolo, 
            Ana Paula Couto, Richard Lobb
			Telecomunication Networks Group
			Politecnico di Torino
			Torino, Italy
			mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file event.h
    Defines the types of events and the EventManager class, which contains
	the methods to schedule events and run the main event loop.
*/

#ifndef _EVENT_H_
#define _EVENT_H_

#include <queue>

#include "types.h"

class EventManager {
public:
	/// The main event-peer_type method -- places an Event in the event queue.
	static void schedule_event(
		EventType type,	///< The type of event to be scheduled
		Time time,			///< The time at which the event is to occur
		PPeer source,		///< Pointer to the source involved (unused for CHUNK_INITIATION)
		PPeer dest,		    ///< Pointer to the destination involved (unused for CHUNK_INITIATION)
		PChunk chunk,		///< Pointer to chunk involved in the event
		size_t id = 0);		///< A user provided event identifier (optional)

	/// An alternative interface to the event-peer_type
	static void schedule_event(
		EventType type,	///< The type of event to be scheduled
		Time time,			///< The time at which the event is to occur
		Path path,			///< The path to which this event relates
		PChunk chunk,		///< Pointer to chunk involved in the event
		size_t id = 0);		///< A user provided event identifier (optional)


	/// Run the simulator
	static void run_event_loop(
		POverlay po	///< A pointer to the overlay on which the simulation is to be run
	);
};

#endif // _SIMULATOR_H_
