/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Csaba Kiraly
  	      Networking Group
  	      Universita di Trento
  	      Trento, Italy
  	      csaba.kiraly@disi.unitn.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	Defines the GNKRegOverlay, a subclass of MBWOverlay which is a
	random k-regular graph.
 */

#include <cmath>
#include "gnkregoverlay.h"
#include "random.h"
#include "globals.h"
#include "misc.h"
#include "peer.h"

// The GNKRegOverlay
// -------------

GNKRegOverlay::GNKRegOverlay(size_t num_peers, size_t degree) {
	if(degree > num_peers) {	//num_peers excludes the source
		fatal("The degree is larger than the total number of peers");
	}
	set_peers(num_peers);  // See MBWOverlay
	set_edges(degree);
}

// Set up each peer with a random neighbourhood of the
// specified degree.

void GNKRegOverlay::set_edges(size_t degree)
{
	size_t n = num_peers();	// includes the source
	unsigned long edge_count=0;
	for (size_t i = 0; i < n; i++) {

		//handle odd*odd cases
		if (edge_count >= n*degree - 1 ) {
			return;
		}

		//verify that there is no saturation
    	size_t cnt = 0;
		for (size_t j = n-1; j > i; j--) {
      			if (peer(j)->num_links_out() < degree && ! find_link (i, j) ) {
        			cnt++;
			}
		}
		if (degree - peer(i)->num_links_out() > cnt) {
			//clear a link
                        int a = (size_t)floor(uniform (0, i, &gSeed));
			int l = (size_t)floor(uniform (0, peer(a)->num_links_out(), &gSeed));
			int b = (*(peer(a)->first_out_link()+l))->to()->id();
      			//fprintf(stderr, " edge- %d %d\n", a,b);
			peer(a)->delete_link(peer(b));
			peer(b)->delete_link(peer(a));
			edge_count-=2;
			i=(a<b?a:b)-1;
			continue;
		}
		while (peer(i)->num_links_out() < degree){		
			size_t j = (size_t)floor(uniform (i+1, n, &gSeed));
			if ( peer(j)->num_links_out() < degree && ! (find_link (i, j)) ) {
      				//fprintf(stderr, " edge+ %d %d\n", i,j);
				add_link (i, j);
				add_link (j, i);
				edge_count+=2;
			}
		}
	}	
}

