/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	A file containing the implementation of the Host class. See host.h
	for the specification for most of the public methods in this file.
 */

#include <limits>
#include "myassert.h"
#include "host.h"
#include "peer.h"
#include "router.h"
#include "globals.h"
#include "event.h"
#include "network.h"

HostType Host::h_host_type = OLD_SIM;
std::string gHostTypeNames[] = {"OLD_SIM", "TWEAKED_OLD_SIM", "PRIORITY_SHARING"};

/// Construct a host with given bandwidth parameters. 
Host::Host(Bandwidth uploadBW, Bandwidth downloadBW,
		   Latency upload_latency, Latency download_latency)
		 : h_id(0), h_router(NULL),
		   h_uploadBW(uploadBW), h_downloadBW(downloadBW),
		   h_upload_latency(upload_latency), h_download_latency(download_latency) {
	 h_download_avail = h_downloadBW;
}

/// Set the ID field of this host to a given value. This is called by the network
/// when a host is added to it. Until that point a host does not have an ID.
void Host::set_id(HostID id) {
	h_id = id;
}

/// Connect this host to a router
void Host::set_router(PRouter router) {
	h_router = router;
}

/// Deliver chunk to attached peer. This is a virtual method, liable to be
/// overridden by subclasses, e.g. to remove any chunk-transmission
/// data that this host is holding, and schedule further chunk deliveries of queued
/// chunks. See the various subclasses for examples.
void Host::deliver_chunk(PChunk chunk, Path path, size_t event_id)
{
	path.to->receive_chunk(chunk, path.from);
}

