/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Maurizio Munafo', Marco Mellia, Gloria Bogliolo, Ana Paula Couto,
		  Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	Implementation of the Host Factory "make_host" method of the Host class.
 */

#include "host.h"
#include "oldsimhost.h"
#include "tweakedoldsimhost.h"
#include "prioritybasedhost.h"
#include "misc.h"

/// ============================================================================
/// The Host factory, called to generate a Host of the type set by the last call
/// to set_host_type.
PHost Host::create_host(Bandwidth uploadBW, Bandwidth downloadBW,
				  Latency upload_latency, Latency download_latency) {
	  PHost phost = NULL;
	  switch (h_host_type) {
		  case OLD_SIM:
			  phost = new OldSimHost(uploadBW, downloadBW, upload_latency, download_latency);
			  break;
		  case TWEAKED_OLD_SIM:
			  phost = new TweakedOldSimHost(uploadBW, downloadBW, upload_latency,download_latency);
			  break;
		  case PRIORITY_SHARING:
			  phost = new PriorityBasedHost(uploadBW, downloadBW, upload_latency,download_latency);
			  break;
		  default:
			  fatal("Unknown host type");
	  }
	  return phost;
}
