/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Maurizio Munafo', Marco Mellia, Gloria Bogliolo, Ana Paula Couto,
		  Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	Implementation of the LatestBA subclass of Peer.
 */

#include "latestbapeer.h"

// ===================== LatestBAPeer ================================
// This peer selects the most recent useful chunk and a random
// target neighbour, weighted by the target's upload bandwidth.
// It is a handcrafted version of the peer behaviour obtained from a 
// config_peer with configuration
// ChunkFirst BestChunk AllChunks Recentness WeightedPeer NeedsSelected UploadBW

void LatestBAPeer::schedule_new_chunk()
{
	if (chunk_trace) printf("T = %lf, peer %d schedule_new_chunk\n", gCurrentTime, id());
	if (num_links_out() > 0 && !p_upload_busy && upload_bandwidth() > 0) {
		bool done = false;
		ChunkIterator it = window()->begin();	// Iterator over chunks, latest first
		while (it != window()->end() && !done) {
			PChunk chunk = *it;
			PLink neighbour = choose_weighted_neighbour(NULL, &needs_chunk_and_not_busy, chunk, &peer_uploadBW);
			if (neighbour != NULL) {
				schedule_chunk(chunk, neighbour);
				done = true;
			}
			else {
				++it;
			}
		}
	}
}
