/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Maurizio Munafo', Marco Mellia, Richard Lobb, Gloria Bogliolo, 
/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

Authors:    Maurizio Munafo', Marco Mellia, Gloria Bogliolo, 
            Ana Paula Couto, Richard Lobb
			Telecomunication Networks Group
			Politecnico di Torino
			Torino, Italy
			mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file link.h
	Defines the Link class, which represents edges in the Overlay class.
 */
#ifndef _LINK_H_
#define _LINK_H_

#include <deque>
#include "types.h"
#include "stats.h"

/// The Link class, representing edges in the network overlay. A link contains
/// the ids of the source and destination (i.e. "from" and "no") peers,
/// a link cost (possibly used for chunk-scheduling) and a LinksStats object
/// in which statistics regarding the link are kept. Note that the distinction
/// between primary link fields and 'statistics' is that the primary fields
/// are used to control the behaviour of the overlay, i.e., the flow of chunks
/// through it, whereas the statistics field could be removed without affecting
/// the diffusion behaviour.
class Link
{
        PPeer l_from;                   ///< The sender peer (not necessary the unique source)
        PPeer l_to;			///< Receiver peer
	LinksStats l_stats;		///< Link statistics (app. dependent, so a separate class)

public:
	/// Constructor, with source and target peers as parameters.
	Link(PPeer from, PPeer to);

	inline PPeer from() {                   ///< The sender peer of the link
	        return l_from;
	};
	inline PPeer to() {			///< The destination peer of the link
		return l_to;
	};

	/// A pointer to the link's LinksStats record, used to maintain
	/// purely statistical information about the link and, say, chunk
	/// flows along it.
	PLinkStats stats();

	/// Clear any simulation or statistics data associated with the link.
	/// Called for each link when the Overlay::clear() method is called.
	void clear();
};

#endif
