/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

Authors:    Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
			Telecomunication Networks Group
			Politecnico di Torino
			Torino, Italy
			mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	Defines the MBWOverlay class, which is an abstract overlay of
	multibandwidth peers. This class defines the set of peers, while
	its subclasses define how those peers are interconnected.
 */

#ifndef _MBW_OVERLAY_H_
#define _MBW_OVERLAY_H_

#include "overlay.h"

/// An MBWOverlay is a overlay with a set of multi-bandwidth peers,
/// randomly allocated to one of 4 different classes. GNROverlay,
/// GNPOverlay and TreeOverlay are subclasses of this overlay, with
/// different edge configurations.
/// MBW_OVERLAYs always have the source as peer 0.
class MBWOverlay : public Overlay {
public:
	MBWOverlay();
	/// Setup the vector of peers to contain num_peers peers (excluding the source).
	/// Each peer is in one of the four different bandwidth classes
	/// defined by the configuration constants in globals.h.
	void set_peers(size_t num_peers);
};

#endif // _MBW_OVERLAY_H_
