/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

Authors:    Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
			Telecomunication Networks Group
			Politecnico di Torino
			Torino, Italy
			mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	Defines the implementation of the base Network class. See Network.h
	for more information.
 */

#include "router.h"
#include "peer.h"
#include "chunk.h"
#include "misc.h"
#include "network.h"
#include "asnetwork.h"
#include "random.h"
#include "globals.h"

NetworkType Network::n_network_type = AS_AWARE_NET;
PNetwork Network::n_current_network = NULL;

/// The strings in this array must exactly match, one for one, the type names defined
/// in the PeerType enum (see peer.h).
std::string gNetworkTypeNames[] = {
	"AS_AWARE_NET",
};

RouterID Network::add_router(PRouter prouter) {
	RouterID id = n_routers.size();
	n_routers.push_back(prouter);
	prouter->set_id(id);
	prouter->set_network(this);
	return id;
}

PRouter Network::router(RouterID router) {
	if (router >= n_routers.size()) {
		fatal("Attempt made to get non-existent router");
	}
	return n_routers[router];
}

HostID Network::register_host(PHost phost) {
	RouterID id = n_hosts.size();
	n_hosts.push_back(phost);
	phost->set_id(id);
	return id;
}

PHost Network::host(HostID host) {
	if (host >= n_hosts.size()) {
		fatal("Attempt made to get non-existent host");
	}
	return n_hosts[host];
}

/// Compute the path latency *within the  network only* of the given path.
/// This is defined as the latency between the two routers only.
/// In the base class, this is zero. Subclasses will probably override this.
Latency Network::network_latency(Path path) {
	return 0;
}

/// Decide whether a chunk being scheduled for delivery should actually
/// be lost in the network.
bool Network::lose_chunk(PChunk chunk, Path path) {
	return uniform(0, 1, &gSeed) < gProbChunkError;
}

// The peer factory
// ================
// Creates a peer of whatever type is specified by Peer::p_peer_type, which
// should have been set by a call to Peer::set_peer_type.

PNetwork Network::make_network()
{
    if(n_current_network==NULL)
    {
        switch (n_network_type)
        {
		case AS_AWARE_NET:
			n_current_network = new ASNetwork();
			break;
		default:
			fatal("Network type not set");
	}
    }
    return n_current_network;
}
