/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

Authors:    Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
			Telecomunication Networks Group
			Politecnico di Torino
			Torino, Italy
			mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
    Define the OldSimHost subclass of Host, which is used to simulate
	the behaviour of the old simulator (which didn't have a network layer).
 */

#ifndef _OLD_SIM_HOST_H_
#define _OLD_SIM_HOST_H_

#include "types.h"
#include "host.h"

/// An OldSimHost is intended to generate an event sequence that more-or-less
/// matches the events generated by the old simulator. Unfortunately, it cannot
/// generate exactly the same sequence, because what was one event in the old simulator
/// is two events in the new one, and the priority queue ordering is undefined with
/// equal-priority events (or appears to behave so, anyway). An attempt to overcome
/// this is made by adding a tiny time offset to the second of the two events; this
/// works for small simulations but will undoubtedly fail for larger ones, leading
/// to a slightly different time sequence of events. The gross properties of the
/// simulation should be the same, however, unless it's a chaotic system!
class OldSimHost : public Host {
public:
	OldSimHost(Bandwidth uploadBW, Bandwidth downloadBW,
		   Latency upload_latency, Latency download_latency);
	virtual void send_chunk(PChunk chunk, Path path);
	virtual void deliver_chunk(PChunk chunk, Path path, size_t event_id);
	virtual void chunk_scheduled(PChunk chunk, Path path);
};

#endif // _OLD_SIM_HOST_H_
