/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Maurizio Munafo', Marco Mellia, Gloria Bogliolo, Ana Paula Couto,
		  Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	Implementation of the Peer Factory "make_peer" method of the Peer class.
 */

#include "types.h"
#include "peer.h"
#include "anapeer.h"
#include "asawarepeer.h"
#include "configpeer.h"
#include "latestrandompeer.h"
#include "latestbapeer.h"
#include "aopeer.h"
#include "roxpeer.h"

// The peer factory
// ================
// Creates a peer of whatever type is specified by Peer::p_peer_type, which
// should have been set by a call to Peer::set_peer_type.

PPeer Peer::make_src(
		Bandwidth upload_bandwidth,		///< The upload bandwidth of the associated host
		Bandwidth download_bandwidth,   ///< The download bandwidth of the associated host  (0 => unlimited)
		size_t peer_class				///< The class (e.g. bandwidth range) of this peer.
										///< Currently used only by AOPeers
	)
{
	PPeer peer = NULL;
	if (p_peer_type == CONFIGURABLE_PEER && config.has_key("ConfigPeerSourceConfig")) {
		peer = new ConfigPeer(upload_bandwidth, download_bandwidth, peer_class, config.getString("ConfigPeerSourceConfig").c_str());
	} else {
		peer = make_peer( upload_bandwidth, download_bandwidth, peer_class);
	}
	return peer;
}


PPeer Peer::make_peer(
		Bandwidth upload_bandwidth,		///< The upload bandwidth of the associated host
		Bandwidth download_bandwidth,   ///< The download bandwidth of the associated host  (0 => unlimited)
		size_t peer_class				///< The class (e.g. bandwidth range) of this peer.
										///< Currently used only by AOPeers
	)
{
	PPeer peer = NULL;
	switch (p_peer_type) {
		case LATEST_RANDOM:
			peer = new LatestRandomPeer( upload_bandwidth, download_bandwidth, peer_class);
			break;
		case LATEST_BA:
			peer = new LatestBAPeer( upload_bandwidth, download_bandwidth, peer_class);
			break;
		case CONFIGURABLE_PEER:
			peer = new ConfigPeer(upload_bandwidth, download_bandwidth, peer_class, config.getString("ConfigPeerConfig").c_str());
			break;
		case AO_PEER:
			peer = new AOPeer(upload_bandwidth, download_bandwidth, peer_class);
			break;
		case ANA:
			peer = new AnaPeer(upload_bandwidth, download_bandwidth, peer_class);
			break;
		case AS_AW_PEER:
			peer = new ASAwarePeer( upload_bandwidth, download_bandwidth, peer_class);
			break;
		case ROX:
			peer = new RoxPeer( upload_bandwidth, download_bandwidth, peer_class);
			break;
		default:
			fatal("Peer type not set");
	}
	return peer;
}
