/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
    Implements the PrebuiltOverlay class, which is an overlay that
	has been output from a previous simulator run.
 */

#include "prebuiltoverlay.h"
#include <iostream>
#include "misc.h"
#include <string>
#include "globals.h"
#include "propertymap.h"
#include "peer.h"

/// A PrebuiltOverlay reads two files in the form of degree.out and links.out
/// to build an overlay. The actual file names are defined in config.txt by the
/// properties InputDegreeFile and InputLinkFile.
PrebuiltOverlay::PrebuiltOverlay() {
	std::string degree_file_name = config.getString("InputDegreeFile");
	std::string links_file_name = config.getString("InputLinkFile");
	std::ifstream degree_file(degree_file_name.c_str());
	if (degree_file.fail()) {
		fatal(std::string("Couldn't open degree file ") + degree_file_name);
	}
	std::ifstream links_file(links_file_name.c_str());
	if (links_file.fail()) {
		fatal(std::string("PrebuiltOverlay: couldn't open links file ") + links_file_name);
	}

	// Loop to read all peer specs

	std::string first_word;
	degree_file >> first_word;
	while (first_word == "Peer" && !degree_file.fail()) {
		int id, degree;
		double upbw, downbw;
		degree_file >> id >> degree >> upbw >> downbw;
		PPeer new_peer = Peer::make_peer(upbw, downbw);
		add_peer(new_peer);
		degree_file >> first_word;
	}

	if (degree_file.fail()) {
		fatal("PrebuiltOverlay: unexpected error on reading degree file");
	}

	// Loop to read links

	int source, destination;
	std::string third_word;
	std::string first_line;
	std::getline(links_file, first_line);  // Ignore the first line
	while (!links_file.fail()) {
		links_file >> first_word >> source >> third_word >> destination;
		if (!links_file.fail()) {
			if (first_word == "Source" && third_word == "Destination") {
				add_link(source, destination);
			}
			else {
				fatal("PrebuiltOverlay: error on reading links file.");
			}
		}
	}
}
