/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	The declaration of the Router class. A Router is the connection
	point of a host to a Network. It currently has no particular
	role, except passing method calls between hosts and the network,
	but is there as a hook for further network-level functionality.
 */

#ifndef _ROUTER_H
#define _ROUTER_H

/// A Router is constructed with the default constructor then attached
/// to a network -- the network sets its id and network fields

#include <vector>
#include "types.h"

class Router
{
	RouterID r_id;       ///< The router ID (defined by the network when the router is connected to it
	PNetwork r_pnetwork;  ///< The network to which the router is connected.
        PAS r_pas;			  ///< The AS ID to which the router is connected.
public:
	void set_id(RouterID id); ///< Set the ID of this router (essentially an index into the network's router table)
        void set_as(PAS as); ///< Set the ID of the AS to which the router is connected
	void set_network(PNetwork network);  ///< Set the network to which this router is attached
	PNetwork network();   ///< Return a pointer to the network to which this router is attached
	HostID add_host(PHost host);		///< add (connect) a host to this router
	/// Return the network latency, i.e. the additional delay (over and above
	/// bandwidth-determined delay) on the given path, between the two routers only.
	Latency network_latency(Path path);
	void sending_chunk(PChunk chunk, Path path);  ///< Notification that a chunk is being sent
	RouterID getId();
};

#endif // _ROUTER_H
