/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Maurizio Munafo', Marco Mellia, Gloria Bogliolo, Ana Paula Couto,
		  Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	Implementation of the LatestBA with IDR and I prioritization subclass of Peer, i.e. Rox PEER
 */

#include "roxpeer.h"
#include <cassert>
#include <map>

// ===================== LatestBAPeer ================================
// This peer selects the most recent useful chunk and a random
// target neighbour, weighted by the target's upload bandwidth.
// It is a handcrafted version of the peer behaviour obtained from a 
// config_peer with configuration
// ChunkFirst BestChunk AllChunks Recentness WeightedPeer NeedsSelected UploadBW


void RoxPeer::schedule_new_chunk()
{
	if (num_links_out() > 0 && !p_upload_busy && upload_bandwidth() > 0) {
                std::map<int, char> TriedChunkPos;
                std::map<int, char> IDRPos;
		bool done = false;
		ChunkIterator it = window()->begin();	// Iterator over chunks, latest first
                int count = 0;
                int countIDR = -1;
                ChunkIterator aux = window()->begin();
                while (aux != window()->end()){
                    countIDR++;
                     if( (*aux)->type() == 'D' || (*aux)->type() == 'I')//|| (*aux)->type() == 'P'|| (*aux)->type() == 'B')
                            IDRPos.insert(std::make_pair(countIDR, '0'));
                    ++aux; 
                }
                //printf("........................................\n");
                countIDR = -1;
                int ritentaIDR = 0;
                aux = window()->begin();
		while ((TriedChunkPos.size() < window()->size()) && !done) {
                    PChunk chunk;
                    int random = rand() % 20; 
                    if (IDRPos.size() != 0 && (random == 0 || ritentaIDR ==1)) { // if aux is pointing to an IDR, give priority to it!
                         // with probability 
                        //printf("IDR  Pos Size %d\n", IDRPos.size());
                            //if (ritentaIDR ==1)
                                //printf("RITENTA \n");
                            //else
                              //  printf("RANDOM\n"); 
                            ritentaIDR = 0;
                            int rand2 = rand() % IDRPos.size();
                                std::map<int, char>::iterator iter2;
                                iter2 = IDRPos.begin();
                                int c;
                                for (c = 0; c < rand2; c++)
                                        ++iter2;
                                countIDR = (iter2->first);
                                //printf("IDR Estratto %d\n",countIDR);
                                for (c = 0; c < (iter2->first); c++)
                                    ++ aux;
                            chunk = *aux;
                            assert(chunk->type() == 'D' || chunk->type() == 'I'|| chunk->type() == 'P'|| (*aux)->type() == 'B');
                    } else {
                        //printf("NON MANDO IDR\n");

                        count = 0;int i;
                        int rand1 = rand() % (window()->size() - TriedChunkPos.size());
                        while(TriedChunkPos.find(count)!= TriedChunkPos.end()){
                                ++it;
                                count ++;
                        }
                        //printf("rand %d    win %d   Tried %d    count %d\n", rand1, window()->size(), TriedChunkPos.size(), count);
                        for(i = 0; i < rand1 ; i++){
                            do{
                                ++it;
                                count ++;
                            }while(TriedChunkPos.find(count)!= TriedChunkPos.end());
                            assert(it != window()->end());
                        } 
                        chunk = *it;
                    }
                        PLink neighbour = NULL;

			         neighbour = choose_weighted_neighbour(NULL, &needs_chunk_and_not_busy, chunk, &peer_uploadBW);
                        
			if (neighbour != NULL) {
				schedule_chunk(chunk, neighbour);
                                //printf("%d    %c\n", chunk->id(), chunk->type());
				done = true;
                                //TriedChunkPos.~map();
			}
			else {
                               // printf("count %d\n", count);
                                TriedChunkPos.insert(std::make_pair(count, '0'));
                                if(chunk->type()=='D'||chunk->type()=='I')//||chunk->type()=='P'|| (*aux)->type() == 'B')
                                    ritentaIDR = 1;
                                it = window()->begin(); aux = window()->begin();
                                if (countIDR != -1){
                                   int ddff = IDRPos.size();
                                   IDRPos.erase(countIDR); 
                                    assert(IDRPos.size() == ddff - 1);
                                    countIDR = -1;
                                }
			}
		}
	}
}
