/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	Implements the TreeOverlay class, a tree-structured directed graph.
 */

#include "treeoverlay.h"
#include "random.h"
#include "globals.h"
#include "misc.h"

TreeOverlay::TreeOverlay(size_t num_peers, size_t degree, size_t max_degree) {
	o_src = 0;          // Source always 0 for TREE
	if(max_degree > num_peers - 1) {
		fatal("The degree is larger than the total number of peers");
	}
	set_peers(num_peers);  // See MBWOverlay
	set_edges(degree, max_degree);
}

// Construct a tree of edges, with peer 0 as root and with poisson-distributed peer degree.
// Each peer is connected to the next un-attached 'degree' peers in peer order.
void TreeOverlay::set_edges (size_t degree, size_t max_degree)
{
	size_t target_peer = 1;

	for(size_t i=0; i < num_peers() && target_peer < num_peers(); i++){
		if(i == target_peer) {  // If we've caught up with the target ... 
			add_link (target_peer-1, i);   // ... we have to avoid disjoint suboverlays
			target_peer++;
		}
		size_t actual_degree = poisson (degree, &gSeed); 
		while((i==0 && actual_degree == 0) || (actual_degree > max_degree)) {
			actual_degree = poisson (degree, &gSeed);
		}
		for(size_t gr=0; gr < actual_degree && target_peer < num_peers(); gr++){
			add_link (i, target_peer);
			target_peer++;
		}
	}
}

