/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file. This header declares all the global datatype names in the simulator.
    The header file for any specific type will generally include
    this file first. Then it will expand in full the particular
    class it is declaring.
 */

#ifndef _TYPES_H_
#define _TYPES_H_

#include <deque>
#include <vector>
#include <set>

class Router;
class Host;
class Network;
class Peer;
class Overlay;
class Link;
class ChunkInstance;
class Chunk;
class Window;
class LinksStats;
class PeerStats;
class PropertyMap;
class VideoStream;
class AS;

struct Path;

typedef Router* PRouter;
typedef Host* PHost;
typedef Network* PNetwork;
typedef Peer* PPeer;
typedef Overlay* POverlay;
typedef Link* PLink;
typedef ChunkInstance* PChunk;
typedef Chunk* PChunkPointer;
typedef Window* PWindow;
typedef LinksStats* PLinkStats;
typedef PeerStats* PPeerStats;
typedef AS* PAS;

typedef std::vector<PPeer> PeerList;
typedef PeerList::iterator PeerIterator;
typedef std::set<PPeer> PeerSet;
typedef PeerSet::iterator PeerSetIterator;

typedef std::deque<PLink>	LinkList;		///< A list (with deque implementation) of PLINKs
typedef LinkList::iterator LinkIterator;	///< An iterator over a LinkList object

//typedef Window::LATEST_FIRST_ITERATOR ChunkIterator;

typedef size_t ChunkID;
typedef size_t HostID;
typedef size_t PeerID;
typedef size_t RouterID;
typedef size_t AS_ID;
typedef double ChunkDeadline;

typedef double ChunkSize;
typedef char   ChunkType;
typedef unsigned FrameRef;
typedef double Bandwidth;  ///< Type used to represent bandwidth (Mbits/s)
typedef double Latency;	   ///< Type used to represent latency (mSec)

/// The type used for simulation time variables.
typedef double Time;
/// Event types
enum EventType {
	CHUNK_INITIATION,	///< Add a chunk to the source's window
	CHUNK_DELIVERED,	///< Process the arrival of a chunk at the destination
	CHUNK_TIMEOUT,		///< A chunk wasn't delivered
	CHUNK_SENT,		///< Process the completion of sending of a chunk by the source
	READY_TO_SEND,		///< extra event to handle problems with event order in when chunks are received and sent simultaneously
	SIMULATION_END		///< end simulation even if there are other events still in the loop. Useful for blind schedulers
};

#endif // _TYPES_H_
