# increase bc's line length to avoid cutting strings
export BC_LINE_LENGTH=150

# copy sources to a new directory, using git or svn (TODO: svn export to be tested)
#  copy_source $PREFIXDIR $TESTCASEFILE
function copy_source {
  (git clone --depth 1 . "$1/src" && git diff | patch -d "$1/src" -p1) ||
    (git clone . "$1/src" && git diff | patch -d "$1/src" -p1) ||
    (svn export . "$1/src" && svn diff | patch -d "$1/src" -p1) ||
    (mkdir "$1/src" && cp *.{c,cpp,h,sh,txt} Makefile "$1/src") ||
    exit 1

  if [[ $2 && -f $2 ]]; then
    cp $2 $1 || exit 1
  fi
}

#  run_in_dir $PREFIXDIR $PREFIX $TESTCASE $PERCENTILE $TESTCASEFILE
function run_in_dir {
  cd $1
  if [[ $3 ]]; then XTRA="$XTRA -t $3"; fi
  if [[ $5 ]]; then XTRA="$XTRA -T $5"; fi
  if [[ $4 ]]; then XTRA="$XTRA -P"; fi
  if [[ $2 ]]; then XTRA="$XTRA -p $2"; fi
echo Executing ./src/test.sh -d . -s src -b $XTRA 
  exec ./src/test.sh -d . -s src -b $XTRA &
  exit 0
}

# cycle through a bunch of variables
# parameters: names of variables to cycle though ... or functions to call
# values for each parameter should be defined in $PARAMNAMEs
# if $PARAMNAMEs doesn't exist, it is assumed to be a function and it is called
function cyclevariables {
  VARs=${1}s	#this variable should contain all the values to cycle through
  if [[ ${!VARs} ]]; then	#if it is a variable, cycle through
    #echo "cycling $1"

    if [[ $# -gt 1 ]] ; then
      for V in ${!VARs} ; do
        #echo $V
	#if [[ $V =~ \\$ ]] ; then
	echo "$V" | grep "\\$" >/dev/null # due to some bash regexp incompatibilities .... we have this ugly way instead of =~
        if [[ $? == 0 ]] ; then
          eval $1=$(eval echo \"scale=6 \; $V\" | bc -l)
        else
          eval $1="$V"
        fi
        cyclevariables ${@:2}
        VARs=${1}s
      done
    fi
  elif [[ $1 == "precall" ]]; then  #call function before
    #echo "calling $2"
    $2
    if [[ $? == 0 && $# -gt 2 ]] ; then        #if precall returned true and there are still variables
      cyclevariables ${@:3}
    fi
  elif [[ $1 == "postcall" ]]; then  #call function before
    if [[ $# -gt 2 ]] ; then
      cyclevariables ${@:3}
    fi
    #echo "calling $2"
    $2
  else
    echo "Error in cycle at $@"
    exit
  fi
}

# convert a parameter's name to its position in the LINEHEAD
# parameters: param's name
function linehead_index {

  i=1
  for p in ${LINEHEAD//\\t/ }; do
    if [[ $p == "\$$1" ]]; then
      echo $i
      return
    fi
    let i++
  done
  echo "Wrong parameter passed to paramter_to_index: $1"
  exit 2
}

# count number of parameters in the LINEHEAD
function linehead_count {
  i=0
  for p in ${LINEHEAD//\\t/ }; do
    let i++
  done
  echo $i
}